import {Component} from '@angular/core';
import {NavigationService} from "@service/navigation.service";
import {FormBuilder, FormGroup, ReactiveFormsModule, Validators} from "@angular/forms";
import SetupService from "@api/v1/setup.service";
import {DatabaseRequest} from "@api/v1/databaseRequest";
import {MatFormFieldModule} from "@angular/material/form-field";
import {MatSelectModule} from "@angular/material/select";
import {MatInput} from "@angular/material/input";
import {MatTooltip} from "@angular/material/tooltip";
import {MatIconButton} from "@angular/material/button";
import {MatIcon} from "@angular/material/icon"
import {MatCheckbox} from "@angular/material/checkbox";

@Component({
  selector: 'app-database',
  standalone: true,
  imports: [
    ReactiveFormsModule,
    MatFormFieldModule,
    MatSelectModule,
    MatInput,
    MatTooltip,
    MatIconButton,
    MatIcon,
    MatCheckbox
  ],
  templateUrl: './database.component.html'
})

export class DatabaseComponent {
  protected databaseForm!: FormGroup;
  protected database = '';
  protected hidePass = true;

  constructor(
    private navigationService: NavigationService, private formBuilder: FormBuilder, private api: SetupService) {
    this.databaseForm = this.formBuilder.group({
      folder: ['', Validators.pattern(/^.{2,}$/)],
      server: ['', Validators.pattern(/^localhost$|^([A-Za-z0-9]+\.)+[A-Za-z]{2,}$|^(?:[0-9]{1,3}\.){3}[0-9]{1,3}$|^([A-Fa-f0-9]{1,4}:){7}[A-Fa-f0-9]{1,4}$|^::1$/)],
      port: ['', Validators.pattern(/^[1-9][0-9]{1,4}$/)],
      database_name: ['', Validators.pattern(/^.{2,}$/)],
      user: ['', Validators.pattern(/^.{2,}$/)],
      password: [''],
      ssl: ['']
    });

    this.databaseForm.get('ssl')?.setValue(false);
    this.navigationService.setNextButtonState(false);
    this.databaseForm.valueChanges.subscribe(() => {
      this.navigationService.setNextButtonState(this.databaseForm.valid);
    });
  }

  private createForm(database: string) {
    if (database === 'sqlite') {
      this.disableControls(['server', 'port', 'database_name', 'user', 'password']);
      this.enableControls(['folder']);
    } else {
      this.enableControls(['server', 'port', 'database_name', 'user', 'password']);
      this.disableControls(['folder']);
    }
  }

  private disableControls(controls: string[]) {
    controls.forEach(control => {
      this.databaseForm.get(control)!.disable({emitEvent: false});
      this.databaseForm.get(control)!.updateValueAndValidity({emitEvent: false});
    });
  }

  private enableControls(controls: string[]) {
    controls.forEach(control => {
      this.databaseForm.get(control)!.enable({emitEvent: false});
      this.databaseForm.get(control)!.updateValueAndValidity({emitEvent: false});
    });
  }

  protected onDatabaseChange(selectedDatabase: string) {
    this.createForm(selectedDatabase);
    this.database = selectedDatabase;

    if (this.database === "sqlite") {
      this.navigationService.nextButtonAction = () => {
        return this.api.setSqlite(this.databaseForm.get('folder')?.value ?? '');
      };
    } else {
      this.navigationService.nextButtonAction = () => {
        let databaseRequest: DatabaseRequest = {
          "server": this.databaseForm.get('server')?.value,
          "port": this.databaseForm.get('port')?.value,
          "database": this.databaseForm.get('database_name')?.value,
          "user": this.databaseForm.get('user')?.value,
          "ssl": this.databaseForm.get('ssl')?.value,
          "password": this.databaseForm.get('password')?.value,
        };
        if (this.database === "mysql")
          return this.api.setMysql(databaseRequest);
        else
          return this.api.setPsql(databaseRequest);
      };
    }
  }

  protected togglePassword(event: MouseEvent) {
    this.hidePass = !this.hidePass;
    event.stopPropagation();
  }
}
